import logging
import os
import toml

from zygoat.constants import Projects
from zygoat.components import Component

log = logging.getLogger()
file_name = "pyproject.toml"


class Black(Component):
    def create(self):
        log.info("Adding black configuration to pyproject.toml")
        backend_file = os.path.join(Projects.BACKEND, file_name)

        with open(backend_file) as f:
            data = toml.load(f)

        data["tool"]["black"] = {"line-length": 95, "target-version": ["py39"]}
        with open(backend_file, "w") as f:
            toml.dump(data, f)


black = Black()



import json
import logging

from zygoat.components import Component
from zygoat.constants import Projects, Phases, Images
from zygoat.utils.shell import docker_run
from zygoat.utils.files import use_dir

log = logging.getLogger()


class BePretty(Component):
    def create(self):
        log.info("Installing be-pretty into frontend project")
        docker_run(["yarn", "add", "--dev", "be-pretty@0.9.5"], Images.NODE, Projects.FRONTEND)

        log.info("Configuring be-pretty to use the correct RC file")
        docker_run(["yarn", "run", "be-pretty", "setDefault"], Images.NODE, Projects.FRONTEND)

        log.info("Formatting project with prettier")
        docker_run(["yarn", "run", "be-pretty", "formatAll"], Images.NODE, Projects.FRONTEND)

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)

    @property
    def installed(self):
        with use_dir(Projects.FRONTEND):
            with open("package.json") as f:
                return "be-pretty" in json.load(f).get("devDependencies", {})


be_pretty = BePretty()



import json
import logging

from zygoat.constants import Projects, Phases, Images
from zygoat.components import Component
from zygoat.utils.shell import docker_run
from zygoat.utils.files import use_dir

from .eslintrc import eslintrc

log = logging.getLogger()


class Eslint(Component):
    dependencies = [
        "eslint",
    ]

    plugins = [
        f"eslint-plugin-{name}"
        for name in ["import", "jest", "jsx-a11y", "prettier", "react", "react-hooks"]
    ]

    configs = [f"eslint-config-{name}" for name in ["airbnb", "prettier"]]

    def create(self):
        log.info("Installing eslint dev dependencies to frontend")
        docker_run(
            ["yarn", "add", "--dev", *self.dependencies],
            Images.NODE,
            Projects.FRONTEND,
        )

        log.info("Installing eslint plugins to frontend")
        docker_run(
            ["yarn", "add", "--dev", *self.plugins],
            Images.NODE,
            Projects.FRONTEND,
        )

        log.info("Installing eslint configs to frontend")
        docker_run(
            ["yarn", "add", "--dev", *self.configs],
            Images.NODE,
            Projects.FRONTEND,
        )

        with use_dir(Projects.FRONTEND):
            log.info("Adding eslint command")
            with open("package.json") as f:
                data = json.load(f)
                data["scripts"]["lint"] = "eslint --ext .js,.jsx ."
                data["scripts"]["fix"] = "yarn lint --fix"

            log.info("Dumping new frontend package file")
            with open("package.json", "w") as f:
                json.dump(data, f)

    def update(self):
        self.call_phase(Phases.CREATE, force_create=True)

    @property
    def installed(self):
        with use_dir(Projects.FRONTEND):
            with open("package.json") as f:
                current = list(json.load(f).get("devDependencies", {}).keys())
                return all(
                    package in current
                    for package in self.dependencies + self.plugins + self.configs
                )


eslint = Eslint(sub_components=[eslintrc])



import shlex
import subprocess
import os

pulled_images = set()


def run(cmd, *args, **kwargs):
    """
    Takes a shell command split into an array and executes it

    :param cmd: An iterable object with strings inside that form a command
    """
    return subprocess.run(
        " ".join([shlex.quote(c) for c in cmd]),
        *args,
        shell=True,
        check=True,
        **kwargs,
    )


def compose_run(cmd, project):
    prelude = ["docker-compose", "run", "--rm", project]
    return run(prelude + cmd)


def docker_run(cmd, image, vol, chown=True, *args, **kwargs):
    if image not in pulled_images:
        run(["docker", "pull", image])
        pulled_images.add(image)

    vol_directory = os.path.join(os.getcwd(), vol)
    prelude = ["docker", "run", "-v", f"{vol_directory}:/data", "-w", "/data", image]
    ret = run(prelude + cmd, *args, **kwargs)

    if chown:
        uid = os.getuid()
        gid = os.getuid()

        run(prelude + ["chown", "-R", f"{uid}:{gid}", "/data"])

    return ret


def multi_docker_run(cmds, *args, **kwargs):
    cmd = "".join([shlex.join(c) + ";" for c in cmds])
    return docker_run(["/bin/bash", "-c", cmd], *args, **kwargs)



from .shell import multi_docker_run
from .files import repository_root
from zygoat.constants import Images, Projects


def install_dependencies(*args, dev=False):
    """
    Installs/upgrades Python dependencies for the backend, and places them in
    the appropriate production or dev requirements files.

    :param args: The packages to install
    :type args: str
    :param dev: Specifies if this is a development or production dependency
    :type dev: bool, optional
    """
    with repository_root():
        add_command = ["poetry", "add"]
        if dev:
            add_command.append("--dev")

        multi_docker_run(
            [
                [
                    "pip",
                    "install",
                    "--upgrade",
                    "pip",
                    "poetry",
                ],
                add_command + list(args),  # args is a tuple and those can't be concatenated
            ],
            Images.PYTHON,
            Projects.BACKEND,
        )



import logging
import os

from zygoat.components import Component, FileComponent
from zygoat.constants import Projects, Images
from zygoat.utils.shell import docker_run

from . import resources

log = logging.getLogger()


class MuiFile(FileComponent):
    resource_pkg = resources
    base_path = os.path.join(Projects.FRONTEND, "pages")
    overwrite = False


class App(MuiFile):
    filename = "_app.js"


class Document(MuiFile):
    filename = "_document.js"


class Mui(Component):
    def create(self):
        log.info("Installing material-ui core and icons")
        docker_run(
            [
                "yarn",
                "add",
                "@mui/material",
                "@mui/icons-material",
                "@emotion/cache",
                "@emotion/react",
                "@emotion/server",
                "@emotion/styled",
            ],
            Images.NODE,
            Projects.FRONTEND,
        )


mui = Mui(sub_components=[App(), Document()])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['zygoat',
 'zygoat.components',
 'zygoat.components.backend',
 'zygoat.components.backend.flake8',
 'zygoat.components.backend.flake8.resources',
 'zygoat.components.backend.resources',
 'zygoat.components.backend.settings',
 'zygoat.components.frontend',
 'zygoat.components.frontend.cypress',
 'zygoat.components.frontend.cypress.resources',
 'zygoat.components.frontend.dependencies',
 'zygoat.components.frontend.dependencies.resources',
 'zygoat.components.frontend.eslint',
 'zygoat.components.frontend.eslint.resources',
 'zygoat.components.frontend.prettier',
 'zygoat.components.frontend.prettier.resources',
 'zygoat.components.frontend.resources',
 'zygoat.components.proxy',
 'zygoat.components.proxy.resources',
 'zygoat.components.resources',
 'zygoat.utils']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0',
 'colorama>=0.4.4,<0.5.0',
 'importlib_metadata>=1.7.0,<2.0.0',
 'python-box[all]>=5.3.0,<6.0.0',
 'redbaron>=0.9.2,<0.10.0',
 'requests>=2.26.0,<3.0.0',
 'rtd-poetry>=0.1.0,<0.2.0',
 'ruamel.yaml>=0.17.10,<0.18.0',
 'semver>=2.13.0,<3.0.0',
 'toml>=0.10.2,<0.11.0',
 'virtualenv>=20.7.0,<21.0.0']

entry_points = \
{'console_scripts': ['zg = zygoat.cli:cli']}

setup_kwargs = {
    'name': 'zygoat',
    'version': '1.16.1',
    'description': '',
    'long_description': '# zygoat\n\n<img src="https://user-images.githubusercontent.com/640862/75250233-e287ea80-57a5-11ea-9d9f-553662a17706.jpeg" />\n\n## What is zygoat?\n\n`zygoat` is a command line tool used to bootstrap and configure a React/Django/Postgres stack web application.\n\nLinting, test configuration, boilerplate, and development environment are automatically taken care of using `zygoat` so that you can get up and running faster.\n\n`zygoat` also includes a preset deployment configuration to allow you to deploy your stack to an AWS environment with a single command. You\'ll get a full serverless AWS stack to keep things inexpensive and nimble.\n\n## How does it work?\n\n`zygoat` works by defining `Components`, defined as parts of projects, and then defining how you implement those components based on whether you\'re creating a new project, updating an existing project, or deleting a component that\'s no longer needed.\n\nFor instance, for the python backend, we want to include `black`, which is a tool for automatically formatting python code in a standard way to make it pep8 compliant. To install `black` in for the python backend part of the project, we create a `Component` for it, specifically a `FileComponent`, which defines how we treat files that we need in projects. Then we register the `Black` component (defined in [black.py](https://github.com/bequest/zygoat/blob/master/zygoat/components/backend/black.py)) with the `Backend` component (defined in [backend/\\_\\_init\\_\\_.py](https://github.com/bequest/zygoat/blob/master/zygoat/components/backend/__init__.py)) as a sub component. This way, whenever you create or update (or delete) a project with the `Backend` component, you\'ll do the same \'phase\' to the `Black` component.\n\n## Installation\n\n```bash\npip install --upgrade zygoat\n```\n\n## Usage\n\n```bash\nmkdir my-cool-new-app && cd my-cool-new-app\ngit init\nzg new my-cool-new-app\n```\n\nFor more customization and configuration, [check out the official documentation](https://zygoat.readthedocs.io/en/latest/).\n\n## How do I develop changes for it?\n\nMake a new git repository somewhere, we\'ll call it test-zg\n\n```bash\nmkdir test-zg && cd test-zg\ngit init\n```\n\nInstall the zygoat package locally\n\n```bash\npip install --user --upgrade ~/Projects/zygoat  # Or wherever you have it\n```\n\nIf you\'re using the asdf version manager, reshim\n\n```bash\nasdf reshim python\n```\n\nRun zg commands, see if they fail\n\n```bash\nzg new test\nzg update\nzg delete\n```\n\n---\n\n## Contributing\n\n`zygoat` is developed using the [Poetry](https://python-poetry.org/docs/) packaging framework for Python projects to make development as simple and portable as possible.\n\n---\n\n## Documentation\n\n[Available on ReadTheDocs](https://zygoat.readthedocs.io/en/latest/)\n',
    'author': 'Bequest, Inc.',
    'author_email': 'oss@willing.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from zygoat.components import FileComponent
from zygoat.constants import Projects

from . import resources


class Eslintrc(FileComponent):
    filename = ".eslintrc.js"
    resource_pkg = resources
    base_path = Projects.FRONTEND


eslintrc = Eslintrc()
